import MersenneTwister from "../../lib/mersenne-twister";

const { ccclass, property } = cc._decorator;

@ccclass
export default class JazzIcon extends cc.Component {
  private generator: MersenneTwister

  @property(cc.Graphics)
  ctx: cc.Graphics = null;

  private shapeCount = 4

  private colors = []

  init(address: string) {
    this.colors = [
      '#01888C', // teal
      '#FC7500', // bright orange
      '#034F5D', // dark teal
      '#F73F01', // orangered
      '#FC1960', // magenta
      '#C7144C', // raspberry
      '#F3C100', // goldenrod
      '#1598F2', // lightning blue
      '#2465E1', // sail blue
      '#F19E02', // gold
    ]
    let seed = Date.now()
    if (address.startsWith("0x")) {
      seed = parseInt(address.slice(2, 10), 16);
    }
    let diameter: number = this.node.width
    this.generator = new MersenneTwister(seed);
    // let remainingColors = this.hueShift(this.colors.slice())
    this.generator.random()
    let remainingColors = this.colors.slice()

    this.ctx.clear()
    this.ctx.fillColor = this.genColor(this.colors)
    this.ctx.fillRect(0, 0 , diameter, diameter)

    for(var i = 0; i < this.shapeCount - 1; i++) {
      this.genShape(remainingColors, diameter, i, this.shapeCount - 1)
    }
  }

  genColor(colors: string[]) {
    let rand = this.generator.random()
    var idx = Math.floor(colors.length * this.generator.random())
    var color = colors.splice(idx,1)[0]
    return new cc.Color().fromHEX(color)
  }

  hueShift(colors: string[]) {
    this.generator.random()
    return colors
  }
  genShape(remainingColors: string[], diameter: number, i: number, total: number) {
    let firstRot = this.generator.random()
    let angle = Math.PI * 2 * firstRot
    let velocity = diameter / total * this.generator.random() + (i * diameter / total)
    let tx = (Math.cos(angle) * velocity)
    let ty = (Math.sin(angle) * velocity)
    let c1 = cc.v2(diameter / 2, diameter / 2)
    let c2 = cc.v2(tx, ty)
  
    let secondRot = this.generator.random()
    let rot = (firstRot * 360) + secondRot * 180
    let angle2 = Math.PI * 2 * rot
    let p0 = this.moveAndRotate(cc.v2(0, 0), c1, c2, angle2, diameter)
    let p1 = this.moveAndRotate(cc.v2(0, diameter), c1, c2, angle2, diameter)
    let p2 = this.moveAndRotate(cc.v2(diameter, diameter), c1, c2, angle2, diameter)
    let p3 = this.moveAndRotate(cc.v2(diameter, 0), c1, c2, angle2, diameter)
    this.ctx.moveTo(p0.x, p0.y)
    this.ctx.lineTo(p1.x, p1.y)
    this.ctx.lineTo(p2.x, p2.y)
    this.ctx.lineTo(p3.x, p3.y)
    let fill = this.genColor(remainingColors)
    this.ctx.fillColor = fill
    this.ctx.fill()
    // this.ctx.stroke()
  }

  moveAndRotate(p1: cc.Vec2, c1: cc.Vec2, c2: cc.Vec2, radians: number, diameter: number) {
    let x1 = c2.x - c1.x
    let y1 = c2.y - c1.y
    let tmpV2 = cc.v2(x1, y1)
    let p2 = p1.add(tmpV2).sub(c2)
    let r = p2.rotate(radians).add(c2)
    let x = Math.min(Math.max(0, r.x), diameter)
    let y = Math.min(Math.max(0, r.y), diameter)
    return cc.v2(x, y)
  }
}
